@@ -42,10 +42,10 @@
 #define E_CELL_SPIN_BUTTON_ARROW_WIDTH  16
 #define PARENT_TYPE e_cell_get_type ()
 
-static void e_cell_spin_button_class_init   (GObjectClass   *klass);
+static void e_cell_spin_button_class_init   (GtkObjectClass   *klass);
 static void e_cell_spin_button_init         (GtkObject        *object);
 
-static void         ecsb_dispose      (GObject	*object);
+static void         ecsb_destroy      (GtkObject	*object);
 
 /* ECell Functions */
 static ECellView *  ecsb_new_view     (ECell                 *ecell,
@@ -120,12 +120,12 @@ static guint    signals[LAST_SIGNAL] = { 0 };
 static ECell   *parent_class;
 
 static void 
-e_cell_spin_button_class_init     (GObjectClass   *klass)
+e_cell_spin_button_class_init     (GtkObjectClass   *klass)
 {
         ECellClass             *ecc    = (ECellClass *) klass;  
 	ECellSpinButtonClass   *ecsbc  = (ECellSpinButtonClass *) klass;
         
-        klass->dispose     = ecsb_dispose;
+        klass->destroy     = ecsb_destroy;
 
 	ecc->realize       = ecsb_realize;
 	ecc->unrealize     = ecsb_unrealize;
@@ -144,18 +144,19 @@ e_cell_spin_button_class_init     (GObjectClass   *klass)
 
 	ecsbc->step        = NULL;
 
-        parent_class       = g_type_class_ref (E_CELL_TYPE);
+        parent_class       = gtk_type_class (E_CELL_TYPE);
 
 	signals[STEP] =
-		g_signal_new ("step",
-			      G_OBJECT_CLASS_TYPE (klass),
-			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (ECellSpinButtonClass, step),
-			      NULL, NULL,
-			      e_marshal_NONE__POINTER_INT_INT_INT,
-			      G_TYPE_NONE,
-			      4, G_TYPE_POINTER, G_TYPE_INT, 
-			      G_TYPE_INT, G_TYPE_INT);
+		gtk_signal_new ("step",
+				GTK_RUN_LAST,
+				E_OBJECT_CLASS_TYPE (klass),
+				GTK_SIGNAL_OFFSET (ECellSpinButtonClass, step),
+				e_marshal_NONE__POINTER_INT_INT_INT,
+				GTK_TYPE_NONE,
+				4, GTK_TYPE_POINTER, GTK_TYPE_INT, 
+				GTK_TYPE_INT, GTK_TYPE_INT);
+	
+	E_OBJECT_CLASS_ADD_SIGNALS (klass, signals, LAST_SIGNAL);
 }
 
 static void 
@@ -364,20 +365,20 @@ ecsb_event        (ECellView        *ecv,
 				/* Yep, which one? */
 				if (event->button.y <= height / 2) {
 					ecsb->up_pressed = TRUE;
-					g_signal_emit (ecsb,
-						       signals[STEP], 0,
-						       ecv,
-						       STEP_UP,
-						       view_col,
-						       row);
+					gtk_signal_emit (GTK_OBJECT(ecsb),
+							 signals[STEP],
+							 ecv,
+							 STEP_UP,
+							 view_col,
+							 row);
 				} else {
 					ecsb->down_pressed = TRUE;
-					g_signal_emit (ecsb,
-						       signals[STEP], 0,
-						       ecv,
-						       STEP_DOWN,
-						       view_col,
-						       row);
+					gtk_signal_emit (GTK_OBJECT(ecsb),
+							 signals[STEP],
+							 ecv,
+							 STEP_DOWN,
+							 view_col,
+							 row);
 				}
 
 				e_table_item_redraw_range (eti,
@@ -512,7 +513,7 @@ ecsb_show_tooltip (ECellView        *ecv,
 }
 
 static void 
-ecsb_dispose (GObject	*object)
+ecsb_destroy (GtkObject	*object)
 {
 	ECellSpinButton *mcsp;
 
@@ -521,7 +522,7 @@ ecsb_dispose (GObject	*object)
 	
 	mcsp = E_CELL_SPIN_BUTTON (object);
 	
-	G_OBJECT_CLASS (parent_class)->dispose (object);
+	GTK_OBJECT_CLASS (parent_class)->destroy (object);
 }
 
 ECell *
@@ -532,15 +533,15 @@ e_cell_spin_button_new (gint     min,
 {
 	ECellSpinButton   *ecsb;
 	
-	ecsb = g_object_new (E_CELL_SPIN_BUTTON_TYPE, NULL);
+	ecsb = gtk_type_new (E_CELL_SPIN_BUTTON_TYPE);
 
 	if (!child_cell) {
 		child_cell = e_cell_number_new (NULL, 
 						GTK_JUSTIFY_LEFT);
 		
-		g_signal_connect (ecsb, "step",
-				  G_CALLBACK (e_cell_spin_button_step),
-				  NULL);
+		gtk_signal_connect (GTK_OBJECT (ecsb), "step",
+				    GTK_SIGNAL_FUNC (e_cell_spin_button_step),
+				    NULL);
 	}
 	
 	ecsb->child   = child_cell;
@@ -559,13 +560,13 @@ e_cell_spin_button_new_float (gfloat    min,
 {
 	ECellSpinButton   *ecsb;
 	
-	ecsb = g_object_new (E_CELL_SPIN_BUTTON_TYPE, NULL);
+	ecsb = gtk_type_new (E_CELL_SPIN_BUTTON_TYPE);
 
 	if (!child_cell) {
 		child_cell = e_cell_float_new (NULL, GTK_JUSTIFY_LEFT);
-		g_signal_connect (ecsb, "step",
-				  G_CALLBACK (e_cell_spin_button_step_float),
-				  NULL);
+		gtk_signal_connect (GTK_OBJECT (ecsb), "step",
+				    GTK_SIGNAL_FUNC (e_cell_spin_button_step_float),
+				    NULL);
 	}
 	
 	ecsb->child   = child_cell;
