@@ -43,6 +43,11 @@
 #include <glade/glade.h>
 #include "gal/util/e-util.h"
 #include "gal/util/e-i18n.h"
+#include "gal/widgets/e-unicode.h"
+
+#include <e-table-scrolled.h>
+#include <e-table-without.h>
+#include <e-table-memory-store.h>
 
 
 #define PARENT_TYPE (gtk_object_get_type())
@@ -56,13 +61,13 @@ enum {
 
 enum {
 	ARG_0,
-	ARG_STATE,
+	ARG_STATE
 };
 
 static guint e_table_config_signals [LAST_SIGNAL] = { 0, };
 
 static void
-config_destroy (GtkObject *object)
+config_finalize (GObject *object)
 {
 	ETableConfig *config = E_TABLE_CONFIG (object);
 
@@ -76,21 +81,15 @@ config_destroy (GtkObject *object)
 		gtk_object_unref (GTK_OBJECT (config->source_spec));
 	config->source_spec = NULL;
 
-	if (config->header)
-		g_free (config->header);
+	g_free (config->header);
 	config->header = NULL;
 
-	GTK_OBJECT_CLASS (config_parent_class)->destroy (object);
-}
-
-static void
-config_finalize (GObject *object)
-{
-	ETableConfig *config = E_TABLE_CONFIG (object);
-
 	g_slist_free (config->column_names);
 	config->column_names = NULL;
-	
+
+	g_free (config->domain);
+	config->domain = NULL;
+
 	G_OBJECT_CLASS (config_parent_class)->finalize (object);
 }
 
@@ -131,7 +130,6 @@ config_class_init (GtkObjectClass *object_class)
 	klass->changed        = NULL;
 
 	object_class->get_arg = config_get_arg;
-	object_class->destroy = config_destroy;
 	G_OBJECT_CLASS (object_class)->finalize = config_finalize;
 
 	e_table_config_signals [CHANGED] =
@@ -146,6 +144,8 @@ config_class_init (GtkObjectClass *object_class)
 
 	gtk_object_add_arg_type ("ETableConfig::state", E_TABLE_STATE_TYPE,
 				 GTK_ARG_READABLE, ARG_STATE);
+
+	glade_gnome_init ();
 }
 
 static ETableColumnSpecification *
@@ -236,7 +236,7 @@ update_sort_and_group_config_dialog (ETableConfig *config, gboolean is_sort)
 				continue;
 			}
 
-			text = gettext (column->title);
+			text = dgettext (config->domain, column->title);
 
 			/*
 			 * Update radio buttons
@@ -292,7 +292,7 @@ config_sort_info_update (ETableConfig *config)
 			continue;
 		}
 		
-		g_string_append (res, gettext ((column)->title));
+		g_string_append (res, dgettext (config->domain, (column)->title));
 		g_string_append_c (res, ' ');
 		g_string_append (
 			res,
@@ -333,8 +333,8 @@ config_group_info_update (ETableConfig *config)
 			g_warning ("Could not find model column in specification");
 			continue;
 		}
-		
-		g_string_append (res, gettext ((column)->title));
+
+		g_string_append (res, dgettext (config->domain, (column)->title));
 		g_string_append_c (res, ' ');
 		g_string_append (
 			res,
@@ -351,6 +351,22 @@ config_group_info_update (ETableConfig *config)
 	g_string_free (res, TRUE);
 }
 
+static void
+setup_fields (ETableConfig *config)
+{
+	int i;
+
+	e_table_without_show_all (config->available_model);
+	e_table_subset_variable_clear (config->shown_model);
+
+	if (config->temp_state) {
+		for (i = 0; i < config->temp_state->col_count; i++) {
+			e_table_subset_variable_add (config->shown_model, config->temp_state->columns[i]);
+			e_table_without_hide (config->available_model, (void *) config->temp_state->columns[i]);
+		}
+	}
+}
+
 static void
 config_fields_info_update (ETableConfig *config)
 {
@@ -358,8 +374,6 @@ config_fields_info_update (ETableConfig *config)
 	GString *res = g_string_new ("");
 	int i;
 
-	return;
-
 	for (i = 0; i < config->state->col_count; i++){
 		for (column = config->source_spec->columns; *column; column++){
 
@@ -368,8 +382,8 @@ config_fields_info_update (ETableConfig *config)
 
 			if (config->state->columns [i] != (*column)->model_col)
 				continue;
-			
-			g_string_append (res, gettext ((*column)->title));
+
+			g_string_append (res, dgettext (config->domain, (*column)->title));
 			if (column [1])
 				g_string_append (res, ", ");
 		}
@@ -406,10 +420,10 @@ do_sort_and_group_config_dialog (ETableConfig *config, gboolean is_sort)
 		case 0:
 			if (is_sort){
 				e_table_sort_info_sorting_truncate (
-					config->state->sort_info, 0);
+					config->temp_state->sort_info, 0);
 			} else {
 				e_table_sort_info_grouping_truncate (
-					config->state->sort_info, 0);
+					config->temp_state->sort_info, 0);
 			}
 			update_sort_and_group_config_dialog (config, is_sort);
 			continue;
@@ -441,21 +455,105 @@ do_sort_and_group_config_dialog (ETableConfig *config, gboolean is_sort)
 		config_group_info_update (config);
 }
 
-GtkWidget *e_table_proxy_etable_new (void);
+static void
+do_fields_config_dialog (ETableConfig *config)
+{
+	int button, running = 1;
+
+	config->temp_state = e_table_state_duplicate (config->state);
+
+	setup_fields (config);
+
+	do {
+		button = gnome_dialog_run (GNOME_DIALOG(config->dialog_show_fields));
+		switch (button){
+			/* OK */
+		case 0:
+			gtk_object_unref (GTK_OBJECT (config->state));
+			config->state = config->temp_state;
+			config->temp_state = 0;
+			running = 0;
+			gnome_property_box_changed (
+				GNOME_PROPERTY_BOX (config->dialog_toplevel));
+			break;
+
+			/* CANCEL */
+		case 1:
+			gtk_object_unref (GTK_OBJECT (config->temp_state));
+			config->temp_state = 0;
+			running = 0;
+			break;
+		}
+		
+	} while (running);
+	gnome_dialog_close (GNOME_DIALOG (config->dialog_show_fields));
+
+	config_fields_info_update (config);
+}
+
+
+ETableMemoryStoreColumnInfo store_columns[] = {
+	E_TABLE_MEMORY_STORE_STRING,
+	E_TABLE_MEMORY_STORE_TERMINATOR
+};
+
+static ETableModel *global_store;  /* Glade better not be reentrant any time soon. */
+
+static void
+create_global_store (ETableConfig *config)
+{
+	int i;
+
+	global_store = e_table_memory_store_new (store_columns);
+	for (i = 0; config->source_spec->columns[i]; i++) {
+		char *text = e_utf8_from_locale_string (dgettext (config->domain,
+								  config->source_spec->columns[i]->title));
+
+		e_table_memory_store_insert_adopt (E_TABLE_MEMORY_STORE (global_store), i, NULL, text);
+	}
+}
+
+char *spec = "<ETableSpecification gettext-domain=\"" E_I18N_DOMAIN "\" no-headers=\"true\" cursor-mode=\"line\" "
+    " draw-grid=\"false\" draw-focus=\"true\" selection-mode=\"browse\">"
+  "<ETableColumn model_col= \"0\" _title=\"Name\" minimum_width=\"30\" resizable=\"true\" cell=\"string\" compare=\"string\"/>"
+  "<ETableState> <column source=\"0\"/>"
+  "<grouping/>"
+  "</ETableState>"
+  "</ETableSpecification>";
+
+GtkWidget *e_table_proxy_etable_shown_new (void);
 
 GtkWidget *
-e_table_proxy_etable_new (void)
+e_table_proxy_etable_shown_new (void)
 {
-	return gtk_label_new ("Field selection dialog not\nimplemented here yet.");
+	ETableModel *model = NULL;
+
+	model = e_table_subset_variable_new (global_store);
+
+	return e_table_scrolled_new (model, NULL, spec, NULL);
+}
+
+GtkWidget *e_table_proxy_etable_available_new (void);
+
+GtkWidget *
+e_table_proxy_etable_available_new (void)
+{
+	ETableModel *model;
+
+	model = e_table_without_new (global_store,
+				     NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+
+	e_table_without_show_all (E_TABLE_WITHOUT (model));
+
+	return e_table_scrolled_new (model, NULL, spec, NULL);
 }
 
 static void
 config_button_fields (GtkWidget *widget, ETableConfig *config)
 {
-	gnome_dialog_run (GNOME_DIALOG(config->dialog_show_fields));
-	gnome_dialog_close (GNOME_DIALOG (config->dialog_show_fields));
+	do_fields_config_dialog (config);
 }
-		
+
 static void
 config_button_sort (GtkWidget *widget, ETableConfig *config)
 {
@@ -506,13 +604,13 @@ configure_dialog (GladeXML *gui, const char *widget_name, ETableConfig *config)
 #endif
 
 static void
-connect_button (ETableConfig *config, GladeXML *gui, const char *widget_name, void *cback)
+connect_button (ETableConfig *config, GladeXML *gui, const char *widget_name, GtkSignalFunc cback)
 {
 	GtkWidget *button = glade_xml_get_widget (gui, widget_name);
 
 	if (button) {
 		gtk_signal_connect (GTK_OBJECT (button), "clicked",
-				    GTK_SIGNAL_FUNC (cback), config);
+				    cback, config);
 	}
 }
 
@@ -524,7 +622,7 @@ sort_entry_changed (GtkEntry *entry, ETableConfigSortWidgets *sort)
 	ETableConfigSortWidgets *base = &config->sort[0];
 	int idx = sort - base;
 	
-	char *s = (char *)gtk_entry_get_text (entry);
+	const char *s = gtk_entry_get_text (entry);
 
 	if (s && s [0] && g_hash_table_lookup (sort->combo->elements, s)){
 		ETableSortColumn c;
@@ -600,9 +698,8 @@ configure_sort_dialog (ETableConfig *config, GladeXML *gui)
 		char *label = l->data;
 
 		for (i = 0; i < 4; i++){
-			gtk_combo_text_add_item (
-				config->sort [i].combo,
-				gettext (label), label);
+			gtk_combo_text_add_item (config->sort [i].combo,
+						 dgettext (config->domain, label), label);
 		}
 	}
 
@@ -629,7 +726,7 @@ group_entry_changed (GtkEntry *entry, ETableConfigSortWidgets *group)
 	ETableSortInfo *sort_info = config->temp_state->sort_info;
 	ETableConfigSortWidgets *base = &config->group[0];
 	int idx = group - base;
-	char *s = (char *) gtk_entry_get_text (entry);
+	const char *s = gtk_entry_get_text (entry);
 
 	if (s && s [0] && g_hash_table_lookup (group->combo->elements, s)){
 		ETableSortColumn c;
@@ -714,7 +811,7 @@ configure_group_dialog (ETableConfig *config, GladeXML *gui)
 		for (i = 0; i < 4; i++){
 			gtk_combo_text_add_item (
 				config->group [i].combo,
-				gettext (label), label);
+				dgettext (config->domain, label), label);
 		}
 	}
 
@@ -734,16 +831,198 @@ configure_group_dialog (ETableConfig *config, GladeXML *gui)
 	}
 }
 
+static void
+add_column (int model_row, gpointer closure)
+{
+	GList **columns = closure;
+	*columns = g_list_prepend (*columns, GINT_TO_POINTER (model_row));
+}
+
+static void
+config_button_add (GtkWidget *widget, ETableConfig *config)
+{
+	GList *columns = NULL;
+	GList *column;
+	int count;
+	int i;
+
+	e_table_selected_row_foreach (config->available, add_column, &columns);
+	columns = g_list_reverse (columns);
+
+	count = g_list_length (columns);
+
+	config->temp_state->columns = g_renew (int, config->temp_state->columns, config->temp_state->col_count + count);
+	config->temp_state->expansions = g_renew (double, config->temp_state->expansions, config->temp_state->col_count + count);
+	i = config->temp_state->col_count;
+	for (column = columns; column; column = column->next) {
+		config->temp_state->columns[i] = e_table_subset_view_to_model_row (E_TABLE_SUBSET (config->available_model), GPOINTER_TO_INT (column->data));
+		config->temp_state->expansions[i] = config->source_spec->columns[config->temp_state->columns[i]]->expansion;
+		i++;
+	}
+	config->temp_state->col_count += count;
+
+	g_list_free (columns);
+
+	setup_fields (config);
+}
+
+static void
+config_button_remove (GtkWidget *widget, ETableConfig *config)
+{
+	GList *columns = NULL;
+	GList *column;
+
+	e_table_selected_row_foreach (config->shown, add_column, &columns);
+
+	for (column = columns; column; column = column->next) {
+		int row = GPOINTER_TO_INT (column->data);
+
+		memmove (config->temp_state->columns + row, config->temp_state->columns + row + 1, sizeof (int) * (config->temp_state->col_count - row - 1));
+		memmove (config->temp_state->expansions + row, config->temp_state->expansions + row + 1, sizeof (double) * (config->temp_state->col_count - row - 1));
+		config->temp_state->col_count --;
+	}
+	config->temp_state->columns = g_renew (int, config->temp_state->columns, config->temp_state->col_count);
+	config->temp_state->expansions = g_renew (double, config->temp_state->expansions, config->temp_state->col_count);
+
+	g_list_free (columns);
+
+	setup_fields (config);
+}
+
+static void
+config_button_up (GtkWidget *widget, ETableConfig *config)
+{
+	GList *columns = NULL;
+	GList *column;
+	int *new_shown;
+	double *new_expansions;
+	int next_col;
+	double next_expansion;
+	int i;
+
+	e_table_selected_row_foreach (config->shown, add_column, &columns);
+	columns = g_list_reverse (columns);
+
+	new_shown = g_new (int, config->temp_state->col_count);
+	new_expansions = g_new (double, config->temp_state->col_count);
+
+	column = columns;
+
+	next_col = config->temp_state->columns[0];
+	next_expansion = config->temp_state->expansions[0];
+
+	for (i = 1; i < config->temp_state->col_count; i++) {
+		if (column && (GPOINTER_TO_INT (column->data) == i)) {
+			new_expansions[i - 1] = config->temp_state->expansions[i];
+			new_shown[i - 1] = config->temp_state->columns[i];
+			column = column->next;
+		} else {
+			new_shown[i - 1] = next_col;
+			next_col = config->temp_state->columns[i];
+
+			new_expansions[i - 1] = next_expansion;
+			next_expansion = config->temp_state->expansions[i];
+		}
+	}
+
+	new_shown[i - 1] = next_col;
+	new_expansions[i - 1] = next_expansion;
+
+	g_free (config->temp_state->columns);
+	g_free (config->temp_state->expansions);
+
+	config->temp_state->columns = new_shown;
+	config->temp_state->expansions = new_expansions;
+
+	g_list_free (columns);
+
+	setup_fields (config);
+}
+
+static void
+config_button_down (GtkWidget *widget, ETableConfig *config)
+{
+	GList *columns = NULL;
+	GList *column;
+	int *new_shown;
+	double *new_expansions;
+	int next_col;
+	double next_expansion;
+	int i;
+
+	e_table_selected_row_foreach (config->shown, add_column, &columns);
+
+	new_shown = g_new (int, config->temp_state->col_count);
+	new_expansions = g_new (double, config->temp_state->col_count);
+
+	column = columns;
+
+	next_col = config->temp_state->columns[config->temp_state->col_count - 1];
+	next_expansion = config->temp_state->expansions[config->temp_state->col_count - 1];
+
+	for (i = config->temp_state->col_count - 1; i > 0; i--) {
+		if (column && (GPOINTER_TO_INT (column->data) == i - 1)) {
+			new_expansions[i] = config->temp_state->expansions[i - 1];
+			new_shown[i] = config->temp_state->columns[i - 1];
+			column = column->next;
+		} else {
+			new_shown[i] = next_col;
+			next_col = config->temp_state->columns[i - 1];
+
+			new_expansions[i] = next_expansion;
+			next_expansion = config->temp_state->expansions[i - 1];
+		}
+	}
+
+	new_shown[0] = next_col;
+	new_expansions[0] = next_expansion;
+
+	g_free (config->temp_state->columns);
+	g_free (config->temp_state->expansions);
+
+	config->temp_state->columns = new_shown;
+	config->temp_state->expansions = new_expansions;
+
+	g_list_free (columns);
+
+	setup_fields (config);
+}
+
+static void
+configure_fields_dialog (ETableConfig *config, GladeXML *gui)
+{
+	config->available = e_table_scrolled_get_table (E_TABLE_SCROLLED (glade_xml_get_widget (gui, "custom-available")));
+	gtk_object_get (GTK_OBJECT (config->available),
+			"model", &config->available_model,
+			NULL);
+
+	config->shown = e_table_scrolled_get_table (E_TABLE_SCROLLED (glade_xml_get_widget (gui, "custom-shown")));
+	gtk_object_get (GTK_OBJECT (config->shown),
+			"model", &config->shown_model,
+			NULL);
+
+	connect_button (config, gui, "button-add",    GTK_SIGNAL_FUNC (config_button_add));
+	connect_button (config, gui, "button-remove", GTK_SIGNAL_FUNC (config_button_remove));
+	connect_button (config, gui, "button-up",     GTK_SIGNAL_FUNC (config_button_up));
+	connect_button (config, gui, "button-down",   GTK_SIGNAL_FUNC (config_button_down));
+
+	setup_fields (config);
+}
+
 static void
 setup_gui (ETableConfig *config)
 {
 	GladeXML *gui;
 
+	create_global_store (config);
+
 	if (e_table_sort_info_get_can_group (config->state->sort_info)) {
-		gui = glade_xml_new (ETABLE_GLADEDIR "/e-table-config.glade", NULL, PACKAGE);
+		gui = glade_xml_new (ETABLE_GLADEDIR "/e-table-config.glade", NULL, E_I18N_DOMAIN);
 	} else {
-		gui = glade_xml_new (ETABLE_GLADEDIR "/e-table-config-no-group.glade", NULL, PACKAGE);
+		gui = glade_xml_new (ETABLE_GLADEDIR "/e-table-config-no-group.glade", NULL, E_I18N_DOMAIN);
 	}
+
+	gtk_object_unref (GTK_OBJECT (global_store));
 	
 	config->dialog_toplevel = glade_xml_get_widget (
 		gui, "e-table-config");
@@ -772,12 +1051,13 @@ setup_gui (ETableConfig *config)
 	config->fields_label = glade_xml_get_widget (
 		gui, "label-fields");
 
-	connect_button (config, gui, "button-sort", config_button_sort);
-	connect_button (config, gui, "button-group", config_button_group);
-	connect_button (config, gui, "button-fields", config_button_fields);
+	connect_button (config, gui, "button-sort", GTK_SIGNAL_FUNC (config_button_sort));
+	connect_button (config, gui, "button-group", GTK_SIGNAL_FUNC (config_button_group));
+	connect_button (config, gui, "button-fields", GTK_SIGNAL_FUNC (config_button_fields));
 	
 	configure_sort_dialog (config, gui);
 	configure_group_dialog (config, gui);
+	configure_fields_dialog (config, gui);
 	
 	gtk_signal_connect (
 		GTK_OBJECT (config->dialog_toplevel), "destroy",
@@ -793,7 +1073,7 @@ setup_gui (ETableConfig *config)
 static void
 config_init (ETableConfig *config)
 {
-	glade_gnome_init ();
+	config->domain = NULL;
 }
 
 ETableConfig *
@@ -818,6 +1098,8 @@ e_table_config_construct (ETableConfig        *config,
 
 	config->state = e_table_state_duplicate (state);
 
+	config->domain = g_strdup (spec->domain);
+
 	for (column = config->source_spec->columns; *column; column++){
 		char *label = (*column)->title;
 
@@ -875,4 +1157,4 @@ e_table_config_raise (ETableConfig *config)
 	gdk_window_raise (GTK_WIDGET (config->dialog_toplevel)->window);
 }
 
-E_MAKE_TYPE(e_table_config, "ETableConfig", ETableConfig, config_class_init, config_init, PARENT_TYPE);
+E_MAKE_TYPE(e_table_config, "ETableConfig", ETableConfig, config_class_init, config_init, PARENT_TYPE)
