@@ -1,5 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/* 
+/*
  * e-table-selection-model.c
  * Copyright 2000, 2001, Ximian, Inc.
  *
@@ -41,7 +41,7 @@ static gint etsm_get_row_count (ESelectionModelArray *esm);
 enum {
 	ARG_0,
 	ARG_MODEL,
-	ARG_HEADER,
+	ARG_HEADER
 };
 
 static void
@@ -236,9 +236,10 @@ etsm_destroy (GtkObject *object)
 
 	etsm = E_TABLE_SELECTION_MODEL (object);
 
-	if (etsm->model_changed_idle_id) {
-		g_source_remove(etsm->model_changed_idle_id);
-	}
+	if (etsm->model_changed_idle_id)
+		g_source_remove (etsm->model_changed_idle_id);
+	etsm->model_changed_idle_id = 0;
+
 	drop_model(etsm);
 	free_hash(etsm);
 
@@ -311,7 +312,7 @@ e_table_selection_model_class_init (ETableSelectionModelClass *klass)
 }
 
 E_MAKE_TYPE(e_table_selection_model, "ETableSelectionModel", ETableSelectionModel,
-	    e_table_selection_model_class_init, e_table_selection_model_init, PARENT_TYPE);
+	    e_table_selection_model_class_init, e_table_selection_model_init, PARENT_TYPE)
 
 /** 
  * e_table_selection_model_new
