@@ -35,7 +35,6 @@
 
 #include <config.h>
 #include "gal/util/e-i18n.h"
-#include "gal/util/e-util.h"
 #include <math.h>
 #include <gnome.h>
 #include "e-group-bar.h"
@@ -48,7 +47,7 @@
 
 static void e_group_bar_class_init (EGroupBarClass *class);
 static void e_group_bar_init (EGroupBar *group_bar);
-static void e_group_bar_dispose (GObject *object);
+static void e_group_bar_destroy (GtkObject *object);
 static void e_group_bar_realize (GtkWidget *widget);
 static void e_group_bar_unrealize (GtkWidget *widget);
 static void e_group_bar_map (GtkWidget *widget);
@@ -57,6 +56,8 @@ static void e_group_bar_size_request (GtkWidget      *widget,
 				      GtkRequisition *requisition);
 static void e_group_bar_size_allocate (GtkWidget *widget,
 				       GtkAllocation *allocation);
+static gint e_group_bar_expose (GtkWidget      *widget,
+				GdkEventExpose *event);
 static void e_group_bar_add (GtkContainer *container,
 			     GtkWidget    *widget);
 static void e_group_bar_remove (GtkContainer     *container,
@@ -103,32 +104,48 @@ static gboolean e_group_bar_auto_show (gpointer data);
 static void e_group_bar_stop_all_animation (EGroupBar	*group_bar);
 
 
-#define PARENT_TYPE GTK_TYPE_CONTAINER
 static GtkContainerClass *parent_class;
 
 
-E_MAKE_TYPE (e_group_bar,
-	     "EGroupBar",
-	     EGroupBar,
-	     e_group_bar_class_init,
-	     e_group_bar_init,
-	     PARENT_TYPE)
+GtkType
+e_group_bar_get_type (void)
+{
+	static GtkType e_group_bar_type = 0;
+
+	if (!e_group_bar_type){
+		GtkTypeInfo e_group_bar_info = {
+			"EGroupBar",
+			sizeof (EGroupBar),
+			sizeof (EGroupBarClass),
+			(GtkClassInitFunc) e_group_bar_class_init,
+			(GtkObjectInitFunc) e_group_bar_init,
+			NULL, /* reserved 1 */
+			NULL, /* reserved 2 */
+			(GtkClassInitFunc) NULL
+		};
+
+		parent_class = gtk_type_class (GTK_TYPE_CONTAINER);
+		e_group_bar_type = gtk_type_unique (GTK_TYPE_CONTAINER,
+						    &e_group_bar_info);
+	}
+
+	return e_group_bar_type;
+}
+
 
 static void
 e_group_bar_class_init (EGroupBarClass *class)
 {
-	GObjectClass *object_class;
+	GtkObjectClass *object_class;
 	GtkWidgetClass *widget_class;
 	GtkContainerClass *container_class;
 
-	object_class = (GObjectClass *) class;
+	object_class = (GtkObjectClass *) class;
 	widget_class = (GtkWidgetClass *) class;
 	container_class = (GtkContainerClass *) class;
 
-	parent_class = g_type_class_ref (PARENT_TYPE);
-
 	/* Method override */
-	object_class->dispose		= e_group_bar_dispose;
+	object_class->destroy		= e_group_bar_destroy;
 
 	widget_class->realize		= e_group_bar_realize;
 	widget_class->unrealize		= e_group_bar_unrealize;
@@ -136,6 +153,7 @@ e_group_bar_class_init (EGroupBarClass *class)
 	widget_class->unmap		= e_group_bar_unmap;
 	widget_class->size_request	= e_group_bar_size_request;
  	widget_class->size_allocate	= e_group_bar_size_allocate;
+	widget_class->expose_event	= e_group_bar_expose;
 
 	container_class->add		= e_group_bar_add;
 	container_class->remove		= e_group_bar_remove;
@@ -174,14 +192,14 @@ e_group_bar_new (void)
 {
 	GtkWidget *group_bar;
 
-	group_bar = GTK_WIDGET (g_object_new (E_GROUP_BAR_TYPE, NULL));
+	group_bar = GTK_WIDGET (gtk_type_new (e_group_bar_get_type ()));
 
 	return group_bar;
 }
 
 
 static void
-e_group_bar_dispose (GObject *object)
+e_group_bar_destroy (GtkObject *object)
 {
 	EGroupBar *group_bar;
 
@@ -192,13 +210,14 @@ e_group_bar_dispose (GObject *object)
 	/* The parent GtkContainer class will automatically destroy all the
 	   child widgets, but it calls gtk_container_foreach() so we must not
 	   destroy our children array until after. */
-	G_OBJECT_CLASS (parent_class)->dispose (object);
+	GTK_OBJECT_CLASS (parent_class)->destroy (object);
 
 	if (group_bar->children)
 	    g_array_free (group_bar->children, TRUE);
 	group_bar->children = NULL;
 }
 
+
 static void
 e_group_bar_realize (GtkWidget *widget)
 {
@@ -236,7 +255,7 @@ e_group_bar_realize (GtkWidget *widget)
 
 	widget->style = gtk_style_attach (widget->style, widget->window);
 	gtk_style_set_background (widget->style, widget->window,
-				  GTK_STATE_ACTIVE);
+				  GTK_STATE_NORMAL);
 
 	gdk_window_set_back_pixmap (widget->window, NULL, FALSE);
 
@@ -501,7 +520,6 @@ e_group_bar_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 				gdk_window_move_resize (group->button_window,
 							0, y, width, button_allocation.height);
 			}
-			gdk_window_raise (group->button_window);
 		}
 		gtk_widget_size_allocate (group->button, &button_allocation);
 
@@ -510,13 +528,10 @@ e_group_bar_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 				gdk_window_resize (group->child_window,
 						   width, child_height);
 				group->child_window_target_y = y + button_allocation.height;
-				gdk_window_raise (group->child_window);
 			} else {
 				gdk_window_move_resize (group->child_window,
 							0, y + button_allocation.height,
 							width, child_height);
-				if (group_num == group_bar->current_group_num)
-					gdk_window_raise (group->child_window);
 			}
 		}
 		gtk_widget_size_allocate (group->child, &child_allocation);
@@ -524,6 +539,81 @@ e_group_bar_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 }
 
 
+static gint
+e_group_bar_expose (GtkWidget      *widget,
+		    GdkEventExpose *event)
+{
+	EGroupBar *group_bar;
+	EGroupBarChild *group;
+	GdkEventExpose child_event;
+	gint group_num;
+
+	g_return_val_if_fail (widget != NULL, FALSE);
+	g_return_val_if_fail (E_IS_GROUP_BAR (widget), FALSE);
+	g_return_val_if_fail (event != NULL, FALSE);
+
+	if (GTK_WIDGET_DRAWABLE (widget)) {
+		group_bar = E_GROUP_BAR (widget);
+
+		child_event = *event;
+
+		for (group_num = 0;
+		     group_num < group_bar->children->len;
+		     group_num++) {
+			group = &g_array_index (group_bar->children,
+						EGroupBarChild, group_num);
+
+			if (event->window == group->button_window
+			    && GTK_WIDGET_DRAWABLE (group->button)
+			    && GTK_WIDGET_NO_WINDOW (group->button)
+			    && gtk_widget_intersect (group->button, &event->area, &child_event.area))
+				gtk_widget_event (group->button, (GdkEvent*) &child_event);
+
+			if (event->window == group->child_window
+			    && GTK_WIDGET_DRAWABLE (group->child)
+			    && GTK_WIDGET_NO_WINDOW (group->child)
+			    && gtk_widget_intersect (group->child, &event->area, &child_event.area))
+				gtk_widget_event (group->child, (GdkEvent*) &child_event);
+		}
+	}
+
+	return FALSE;
+}
+
+#if 0
+static void
+e_group_bar_draw (GtkWidget    *widget,
+		  GdkRectangle *area)
+{
+	EGroupBar *group_bar;
+	EGroupBarChild *group;
+	gint group_num;
+	GdkRectangle child_area;
+
+	g_return_if_fail (widget != NULL);
+	g_return_if_fail (E_IS_GROUP_BAR (widget));
+   
+	if (GTK_WIDGET_DRAWABLE (widget)) {
+		group_bar = E_GROUP_BAR (widget);
+	
+		for (group_num = 0;
+		     group_num < group_bar->children->len;
+		     group_num++) {
+			group = &g_array_index (group_bar->children,
+						EGroupBarChild, group_num);
+
+			if (GTK_WIDGET_DRAWABLE (group->button)
+			    && gtk_widget_intersect (group->button, area, &child_area))
+				gtk_widget_draw (group->button, &child_area);
+
+			if (GTK_WIDGET_DRAWABLE (group->child)
+			    && gtk_widget_intersect (group->child, area, &child_area))
+				gtk_widget_draw (group->child, &child_area);
+		}
+	}
+}
+#endif
+
 static void
 e_group_bar_add (GtkContainer *container,
 		 GtkWidget    *widget)
@@ -582,9 +672,6 @@ e_group_bar_forall (GtkContainer   *container,
 
 	group_bar = E_GROUP_BAR (container);
 
-	if (group_bar->children == NULL)
-		return;
-
 	/* Note that drag-and-drop does not check the Z-order of widgets, so
 	   we have to iterate through them from top to bottom, or it will
 	   not work properly. We also have to use temporary lists so widgets
@@ -897,13 +984,13 @@ e_group_bar_add_group		(EGroupBar	*group_bar,
 		}
 	}
 
-	g_signal_connect (group->button, "clicked",
-			  G_CALLBACK (e_group_bar_on_button_clicked),
-			  group_bar);
-	
-	g_signal_connect (group->button, "drag_leave",
-			  G_CALLBACK (e_group_bar_on_button_drag_leave),
-			  group_bar);
+	gtk_signal_connect (GTK_OBJECT (group->button), "clicked",
+			    GTK_SIGNAL_FUNC (e_group_bar_on_button_clicked),
+			    group_bar);
+
+	gtk_signal_connect (GTK_OBJECT (group->button), "drag_leave",
+			    GTK_SIGNAL_FUNC (e_group_bar_on_button_drag_leave),
+			    group_bar);
 
 	/* Set up the button as a drop target.  */
 
@@ -911,9 +998,9 @@ e_group_bar_add_group		(EGroupBar	*group_bar,
 			   0, NULL, 0,
 			   GDK_ACTION_COPY | GDK_ACTION_MOVE);
 
-	g_signal_connect (group->button, "drag_motion",
-			  G_CALLBACK (e_group_bar_on_button_drag_motion),
-			  group_bar);
+	gtk_signal_connect (GTK_OBJECT (group->button), "drag_motion",
+			    GTK_SIGNAL_FUNC (e_group_bar_on_button_drag_motion),
+			    group_bar);
 	return group_num;
 }
 
@@ -1463,15 +1550,13 @@ e_group_bar_stop_all_animation	(EGroupBar	*group_bar)
 		group_bar->auto_show_timeout_id = 0;
 	}
 
-	if (group_bar->children != NULL) {
-		for (group_num = 0;
-		     group_num < group_bar->children->len;
-		     group_num++) {
-			group = &g_array_index (group_bar->children,
-						EGroupBarChild, group_num);
-			group->button_window_in_animation = FALSE;
-			group->child_window_in_animation = FALSE;
-		}
+	for (group_num = 0;
+	     group_num < group_bar->children->len;
+	     group_num++) {
+		group = &g_array_index (group_bar->children,
+					EGroupBarChild, group_num);
+		group->button_window_in_animation = FALSE;
+		group->child_window_in_animation = FALSE;
 	}
 }
 
