@@ -30,10 +30,9 @@
 extern "C" {
 #endif /* __cplusplus */
 
-#define GTK_COMBO_BOX_TYPE          (gtk_combo_box_get_type())
-#define GTK_COMBO_BOX(obj)	    G_TYPE_CHECK_INSTANCE_CAST (obj, gtk_combo_box_get_type (), GtkComboBox)
-#define GTK_COMBO_BOX_CLASS(klass)  G_TYPE_CHECK_CLASS_CAST (klass, gtk_combo_box_get_type (), GtkComboBoxClass)
-#define GTK_IS_COMBO_BOX(obj)       G_TYPE_CHECK_INSTANCE_TYPE (obj, gtk_combo_box_get_type ())
+#define GTK_COMBO_BOX(obj)	    GTK_CHECK_CAST (obj, gtk_combo_box_get_type (), GtkComboBox)
+#define GTK_COMBO_BOX_CLASS(klass)  GTK_CHECK_CLASS_CAST (klass, gtk_combo_box_get_type (), GtkComboBoxClass)
+#define GTK_IS_COMBO_BOX(obj)       GTK_CHECK_TYPE (obj, gtk_combo_box_get_type ())
 
 typedef struct _GtkComboBox	   GtkComboBox;
 typedef struct _GtkComboBoxPrivate GtkComboBoxPrivate;
