@@ -45,7 +45,7 @@ enum {
 	LAST_SIGNAL
 };
 
-static gint color_combo_signals [LAST_SIGNAL] = { 0, };
+static guint color_combo_signals [LAST_SIGNAL] = { 0, };
 
 static GObjectClass *color_combo_parent_class;
 
@@ -67,6 +67,15 @@ color_combo_set_color_internal (ColorCombo *cc, GdkColor *color)
 			       NULL);
 }
 
+static void
+color_combo_destroy (GtkObject *object)
+{
+	GtkObjectClass *klass = (GtkObjectClass *)color_combo_parent_class;
+
+	if (klass->destroy)
+                klass->destroy (object);
+}
+
 static void
 color_combo_finalize (GObject *object)
 {
@@ -77,6 +86,7 @@ static void
 color_combo_class_init (GObjectClass *object_class)
 {
 	object_class->finalize = color_combo_finalize;
+	((GtkObjectClass *)object_class)->destroy = color_combo_destroy;
 
 	color_combo_parent_class = gtk_type_class (gtk_combo_box_get_type ());
 
@@ -198,6 +208,7 @@ color_combo_construct (ColorCombo *cc, GdkPixbuf *icon,
 		       char const *no_color_label,
 		       ColorGroup *color_group)
 {
+	GdkColor *color;
 	g_return_if_fail (cc != NULL);
 	g_return_if_fail (IS_COLOR_COMBO (cc));
 
@@ -256,8 +267,9 @@ color_combo_construct (ColorCombo *cc, GdkPixbuf *icon,
 				 cc->preview_button,
 				 GTK_WIDGET (cc->palette));
 
-	color_combo_set_color_internal (cc,
-		color_palette_get_current_color (cc->palette, NULL));
+	color = color_palette_get_current_color (cc->palette, NULL);
+	color_combo_set_color_internal (cc, color);
+	if (color) gdk_color_free (color);
 }
 
 /* color_combo_get_color:
